/*
	Federico Balaguer
	
 */

package observations;

import java.lang.*;
import java.io.*;
import java.util.*;
import java.net.URL;
import java.net.MalformedURLException;

import org.xml.sax.Parser;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.ParserFactory;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Element;

public class ObservationRepository 
{
	Hashtable observationTypes = new Hashtable();


public Observation observationForPhenomenon(String phenomenon) throws Exception
{
	return (Observation)retrieve(phenomenon).createObservation()  ;
}

private ObservationFactory retrieve(String phenomenon)
        throws Exception
{
  if (observationTypes.containsKey(phenomenon))
    return (ObservationFactory)observationTypes.get(phenomenon);  
  return (ObservationFactory)loadObservationType(phenomenon+".xml");
}

	  
public ObservationType loadObservationType(String filename)
{
  String xmlFile = null;
  URL u = null;
  boolean warmup = false;
  org.w3c.dom.Document xmlDoc=null; 
  
   try 
   {
	 u = createURL(filename);	            
    } 
    catch (Exception e) 
    {
	System.out.println(e);
            System.exit(-1);
    }
   xmlFile = u.toString();
   xmlDoc = ParseXMLFile(xmlFile);
   return (ObservationType)retrieveObservationType((Element)xmlDoc.getDocumentElement());
}

public Validator loadValidator(String filename)
{
  String xmlFile = null;
  URL u = null;
  boolean warmup = false;
  org.w3c.dom.Document xmlDoc=null; 
  
   try 
   {

	 u = createURL(filename);	            
    } 
    catch (Exception e) 
    {	System.out.println("loadValidator: "+e);
	System.exit(-1); }

   xmlFile = u.toString();
   xmlDoc = ParseXMLFile(xmlFile);
   return (Validator) retrieveValidator((Element)xmlDoc.getDocumentElement());
}

private org.w3c.dom.Document ParseXMLFile(String xmlFile)
{
   String parserClass = "com.ibm.xml.parsers.NonValidatingDOMParser";
   Parser parser=null;
   try 
   { parser = ParserFactory.makeParser(parserClass); 
     parser.parse(xmlFile); 
   } 
   catch (SAXException sx) 
   {
      Exception ex = sx.getException();
      if (ex != null)
        ex.printStackTrace();
      else
        sx.printStackTrace();
      System.exit(-1);
   }
   catch ( java.lang.InstantiationException ex )
   { System.out.println(ex);
   }
   catch (java.lang.IllegalAccessException ex)
   { System.out.println(ex);
   }
   catch (java.io.IOException ex)	
   { System.out.println(ex);
   }
   catch (java.lang.ClassNotFoundException ex)
   { System.out.println(ex);
   }
   return ((com.ibm.xml.parsers.NonValidatingDOMParser)parser).getDocument();
}
 
 
public Validator retrieveValidator(org.w3c.dom.Element element )
{ 
  Validator validator;	
  Hashtable metadata = new Hashtable();
  Vector tags = new Vector();
  validatorTags(tags);
  XMLMetadataReader.traverse(element, metadata,tags) ;
 
  String className = (String) metadata.get("Class");	
  String name = (String) metadata.get("Name");
  String descString = (String) metadata.get("Descriptors");

  if (className.equals("DiscreteValidator"))
    return new DiscreteValidator( (Set)DiscreteValidator.descriptors(descString),name);

  if (className.equals("RangeValidator"))
    return new RangeValidator( RangeValidator.descriptors(descString),name);

  return new DefaultValidator();       
}

private void validatorTags(Vector filters)
{
  filters.addElement("Class");
  filters.addElement("Name");
  filters.addElement("Descriptors");
}

private void observationTypeTags(Vector filters)
{
  filters.addElement("Class");
  filters.addElement("Name");
  filters.addElement("Validator");
  filters.addElement("Components");
}

private ObservationType retrieveObservationType(org.w3c.dom.Element element )
{   
  DefaultValidator validator = new DefaultValidator();
  ObservationType observationType = new PrimitiveObservationType((Validator) validator,"Error");
  Hashtable metadata = new Hashtable();
  Vector tags = new Vector();
  observationTypeTags(tags);
  XMLMetadataReader.traverse(element, metadata,tags) ;
  String className = (String) metadata.get("Class");
  String name = (String) metadata.get("Name");

  
  
  if (className.equals("PrimitiveObservationType"))
   { 
     Validator v = validator((String) metadata.get("Validator"));
     observationType =  new PrimitiveObservationType(v,name);
   }
  if (className.equals("CompositeObservationType"))
   {
     
    String components = (String) metadata.get("Components");
     observationType = new CompositeObservationType(observationTypes(components), name);
   }
  observationTypes.put(name, observationType);
  return observationType;       
}

public Validator validator(String aName)
{ 
  return loadValidator(aName+".xml");
}

private  Vector observationTypes(String formattedString)
{
  Vector components = new Vector();
  Vector names = Tokenizer.substrings(formattedString,",");
  Enumeration enum =  names.elements();
  while(enum.hasMoreElements())
  { String name = (String)enum.nextElement();
    components.addElement(observationTypes.get(name));
  }
  return new Vector();
}

public URL createURL(String name) throws Exception 
{
        try {
            URL u = new URL(name);
            return u;
        } catch (MalformedURLException ex) {
        }
        URL u = new URL("file:" + new File(name).getAbsolutePath());
        return u;
}
   
}
